import os
from pathlib import Path

from secrets_vault import SecretsVault, exceptions

BASE_DIR = Path(__file__).parent
TEST_DATA_DIR = BASE_DIR / "test-data"


def test_create():
    vault, master_key = SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-1.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-1.key",
    )

    assert os.path.exists(TEST_DATA_DIR / "secrets-1.json.enc")
    assert os.path.exists(TEST_DATA_DIR / "master-1.key")
    assert master_key is not None and len(master_key) == 44
    with open(TEST_DATA_DIR / "master-1.key", "rb") as fin:
        assert fin.read().decode() == master_key

    vault = SecretsVault(
        secrets_filepath=TEST_DATA_DIR / "secrets-1.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-1.key",
    )
    assert vault.get("my-password") == "supersecret"


def test_requires_master_key():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-2.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-2.key",
    )

    try:
        SecretsVault(
            secrets_filepath=TEST_DATA_DIR / "secrets-2.json.enc",
            master_key_filepath=TEST_DATA_DIR / "master-456.key",
        )
        assert False, "Should throw"
    except Exception as e:
        assert isinstance(e, exceptions.MasterKeyNotFound)


def test_provides_master_key_via_env_var():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-2a.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-2a.key",
    )

    with open(TEST_DATA_DIR / "master-2a.key", "rb") as fin:
        master_key = fin.read().decode()
        os.environ["MASTER_KEY"] = master_key

    vault = SecretsVault(
        secrets_filepath=TEST_DATA_DIR / "secrets-2a.json.enc",
        master_key_filepath=None,
    )
    assert vault.get("my-password") == "supersecret"

    # Cleanup to not affect other tests
    del os.environ["MASTER_KEY"]


def test_requires_secrets_file():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-3.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-3.key",
    )

    try:
        SecretsVault(
            secrets_filepath=TEST_DATA_DIR / "secrets-456.json.enc",
            master_key_filepath=TEST_DATA_DIR / "master-3.key",
        )
        assert False, "Should throw"
    except Exception as e:
        assert isinstance(e, exceptions.SecretsFileNotFound)


def test_save():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-4.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-4.key",
    )

    vault = SecretsVault(
        secrets_filepath=TEST_DATA_DIR / "secrets-4.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-4.key",
    )

    assert vault.get("my-password") == "supersecret"
    assert vault.get("hello") is None
    assert vault.get("nested") is None

    vault.set("hello", "world")
    vault.set("nested", {"object": "value"})
    vault.save()

    assert vault.get("hello") == "world"
    assert vault.get("nested") == {"object": "value"}


def test_large_secrets_file():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-5.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-5.key",
    )

    vault = SecretsVault(
        secrets_filepath=TEST_DATA_DIR / "secrets-5.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-5.key",
    )

    for i in range(10000):
        vault.set(f"key-{i}", f"value-{i}")
    vault.save()

    for i in range(10000):
        assert vault.get(f"key-{i}") == f"value-{i}"


def test_wrong_key():
    SecretsVault.create(
        secrets_filepath=TEST_DATA_DIR / "secrets-6.json.enc",
        master_key_filepath=TEST_DATA_DIR / "master-6.key",
    )

    try:
        SecretsVault(
            secrets_filepath=TEST_DATA_DIR / "secrets-6.json.enc",
            master_key_filepath=TEST_DATA_DIR / "master-5.key",
        )
        assert False, "Should throw"
    except Exception as e:
        assert isinstance(e, exceptions.MasterKeyInvalid)



import json
import logging
import os
import tempfile
from pathlib import Path
from subprocess import call

from cryptography.fernet import Fernet, InvalidToken

from secrets_vault.constants import (
    DEFAULT_MASTER_KEY_FILEPATH,
    DEFAULT_SECRETS_FILEPATH,
)
from secrets_vault.exceptions import (
    MasterKeyNotFound,
    MasterKeyInvalid,
    SecretsFileNotFound,
    SecretsFileAlreadyExists,
)

log = logging.getLogger(__name__)


class SecretsVault:
    def __init__(
        self,
        master_key=None,
        secrets_filepath=DEFAULT_SECRETS_FILEPATH,
        master_key_filepath=DEFAULT_MASTER_KEY_FILEPATH,
    ):
        if master_key is None:
            master_key = self._load_master_key(master_key_filepath)
        try:
            self.fernet = Fernet(master_key)
            self.secrets_filename = secrets_filepath
            self.secrets = dict()
            self._load_secrets()
        except InvalidToken:
            raise MasterKeyInvalid("Master key is malformed or invalid")

    @staticmethod
    def create(
        secrets_filepath=DEFAULT_SECRETS_FILEPATH,
        master_key_filepath=DEFAULT_MASTER_KEY_FILEPATH,
    ):
        """
        Create a new secrets file and returns the master key - keep it safe!
        """
        if Path(secrets_filepath).exists():
            raise SecretsFileAlreadyExists(f"Secrets file {secrets_filepath} already exists")

        log.info(f"Creating new secrets file {secrets_filepath}")
        Path(secrets_filepath).touch()

        master_key = Fernet.generate_key().decode()
        with open(master_key_filepath, "w") as fout:
            fout.write(master_key)

        vault = SecretsVault(master_key, secrets_filepath)
        vault.set("my-user", "foo")
        vault.set("my-password", "supersecret")
        vault.save()

        return vault, master_key

    def require(self, key):
        return self.secrets[key]

    def get(self, key, default=None):
        return self.secrets.get(key, default)

    def set(self, key, value):
        self.secrets[key] = value

    def delete(self, key):
        if key in self.secrets:
            del self.secrets[key]

    def edit_secrets(self):
        """
        Decrypts and opens the secrets file in an editor. On save, the file is encrypted again.
        """
        self._load_secrets()
        EDITOR = os.environ.get("EDITOR", "vim").split(" ")  # 'could be "code --wait"'
        with tempfile.NamedTemporaryFile(suffix=".tmp.yml") as tf:
            tf.write(self._serialize())
            tf.flush()
            call([*EDITOR, tf.name])
            tf.seek(0)
            newsecrets = json.loads(tf.read())

        if self.secrets == newsecrets:
            log.info("No changes applied")
            return
        self.secrets = newsecrets
        self.save()

    def save(self):
        with open(self.secrets_filename, "wb") as fout:
            fout.write(self.fernet.encrypt(self._serialize()))
        log.info(f"Wrote encrypted secrets to {self.secrets_filename}")

    @staticmethod
    def _load_master_key(master_key_filepath=None):
        master_key = os.environ.get("MASTER_KEY")
        if not master_key and master_key_filepath and os.path.exists(master_key_filepath):
            with open(Path(master_key_filepath).absolute()) as fin:
                master_key = fin.read().strip()
        if not master_key:
            raise MasterKeyNotFound(
                "Could not find encryption master key. "
                f"Set it as an environment variable 'MASTER_KEY', or in a file '{master_key_filepath}'"
            )
        return master_key

    def _load_secrets(self):
        log.info(f"Loading encrypted secrets from {self.secrets_filename}")
        if not os.path.exists(self.secrets_filename):
            raise SecretsFileNotFound(f"Could not find secrets file {self.secrets_filename}")
        with open(self.secrets_filename, "r") as fin:
            contents = fin.read()
            if contents:
                self.secrets = json.loads(self.fernet.decrypt(contents))
            else:
                self.secrets = dict()

    def _serialize(self):
        return json.dumps(self.secrets, sort_keys=False, indent=4).encode()



#!/usr/bin/env python
import pathlib
import sys

from setuptools import find_packages, setup

assert sys.version_info >= (3, 9), "Requires Python v3.9 or above."

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
REQUIREMENTS = [
    x.strip() for x in (HERE / "requirements.txt").read_text().split("\n") if not x.startswith("#") and x.strip()
]

setup(
    name="secrets-vault",
    version="0.1.6",
    author="Anthony N. Simon",
    url="https://github.com/anthonynsimon/secrets-vault",
    description="Simple encrypted secrets for Python",
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "secrets-vault=secrets_vault.__main__:cli",
            "secrets=secrets_vault.__main__:cli",
        ]
    },
    install_requires=REQUIREMENTS,
    tests_require=[],
)



class SecretsStoreException(Exception):
    pass


class MasterKeyNotFound(SecretsStoreException):
    pass


class MasterKeyInvalid(SecretsStoreException):
    pass


class SecretsFileNotFound(SecretsStoreException):
    pass


class SecretsFileAlreadyExists(SecretsStoreException):
    pass



import json

import click

from secrets_vault import SecretsVault, exceptions, constants

common_options = [
    click.option(
        "-s",
        "--secrets-filepath",
        default=constants.DEFAULT_SECRETS_FILEPATH,
        help="Path to the encrypted secrets vault.",
    ),
    click.option(
        "-m",
        "--master-key-filepath",
        default=constants.DEFAULT_MASTER_KEY_FILEPATH,
        help="Path to the master.key file.",
    ),
]


def add_options(options):
    def _add_options(func):
        for option in options:
            func = option(func)
        return func

    return _add_options


def serialize(v):
    if not v:
        return ""
    if isinstance(v, str):
        return v
    if isinstance(v, int):
        return str(v)
    if isinstance(v, float):
        return str(v)
    return json.dumps(v, default=str, sort_keys=False)


@click.group(help="Manage a local secrets vault.")
@add_options(common_options)
def cli(**kwargs):
    pass


@cli.command(
    help="Generate a new secrets vault and master.key pair. If a secrets vault already exists, this will abort."
)
@add_options(common_options)
def init(**kwargs):
    try:
        SecretsVault.create(
            secrets_filepath=kwargs["secrets_filepath"], master_key_filepath=kwargs["master_key_filepath"]
        )
        click.echo(f"Generated new secrets vault at {kwargs['secrets_filepath']}")
        click.echo(f"Generated new master key at {kwargs['master_key_filepath']}")
    except exceptions.SecretsFileAlreadyExists:
        print("Secrets file already exists, aborting...")
        exit(1)


def with_vault(func, secrets_filepath, master_key_filepath):
    try:
        vault = SecretsVault(secrets_filepath=secrets_filepath, master_key_filepath=master_key_filepath)
        func(vault)
        exit(0)
    except exceptions.MasterKeyNotFound:
        click.echo(
            f"No master key found. Set it via the environment variable 'MASTER_KEY', or in a file at '{constants.DEFAULT_MASTER_KEY_FILEPATH}'"
        )
    except exceptions.SecretsFileNotFound:
        click.echo("Secrets file already exists, aborting...")
    exit(1)


@cli.command(
    help="Get one or more secret values. If none are specified, all secrets are printed (eg. `secrets get`). You can also provide multiple keys to retrive more than one secret at a time (eg. secrets get foo1 foo2 foo3)."
)
@add_options(common_options)
@click.argument("key", required=False, nargs=-1)
def get(key, **kwargs):
    def handler(vault):
        if key:
            if len(key) == 1:
                click.echo(serialize(vault.get(key[0])))
            else:
                for k in key:
                    click.echo(f"{k}: {serialize(vault.get(k))}")
        else:
            for k, v in vault.secrets.items():
                click.echo(f"{k}: {serialize(v)}")

    with_vault(
        handler,
        secrets_filepath=kwargs["secrets_filepath"],
        master_key_filepath=kwargs["master_key_filepath"],
    )


@cli.command(
    help="Prints a provided secret key as one or more env variables. In case the value is a nested object, it will flatten the key=value pairs."
)
@add_options(common_options)
@click.argument("key")
def envify(key, **kwargs):
    def handler(vault):
        value = vault.get(key)
        if isinstance(value, dict):
            for k, v in value.items():
                click.echo(f"{k}={serialize(v)}")
        else:
            click.echo(f"{key}={serialize(value)}")

    with_vault(
        handler,
        secrets_filepath=kwargs["secrets_filepath"],
        master_key_filepath=kwargs["master_key_filepath"],
    )


@cli.command(
    help="Store a secret. If the secret already exists, it will be overwritten. For example: `secrets set foo bar`"
)
@add_options(common_options)
@click.argument("key")
@click.argument("value")
def set(key, value, **kwargs):
    def handler(vault):
        vault.set(key, value)
        vault.save()

    with_vault(
        handler,
        secrets_filepath=kwargs["secrets_filepath"],
        master_key_filepath=kwargs["master_key_filepath"],
    )


@cli.command("del", help="Delete a secret. For example: `secrets del foo`")
@add_options(common_options)
@click.argument("key")
def delete(key, **kwargs):
    def handler(vault):
        vault.delete(key)
        vault.save()

    with_vault(
        handler,
        secrets_filepath=kwargs["secrets_filepath"],
        master_key_filepath=kwargs["master_key_filepath"],
    )


@cli.command(help="Open the secrets vault in your configured $EDITOR.")
@add_options(common_options)
def edit(**kwargs):
    def handler(vault):
        vault.edit_secrets()

    with_vault(
        handler,
        secrets_filepath=kwargs["secrets_filepath"],
        master_key_filepath=kwargs["master_key_filepath"],
    )


if __name__ == "__main__":
    cli()
